import time
from dataclasses import field
from typing import List, Any, Union, Dict

from openai.types.chat import ChatCompletionMessageParam, ChatCompletionMessage
from rich.panel import Panel

from hackingBuddyGPT.capabilities import Capability
from hackingBuddyGPT.capabilities.http_request import HTTPRequest
from hackingBuddyGPT.capabilities.submit_flag import SubmitFlag
from hackingBuddyGPT.usecases.agents import Agent
from hackingBuddyGPT.usecases.base import AutonomousAgentUseCase, use_case
from hackingBuddyGPT.utils import LLMResult, tool_message
from hackingBuddyGPT.utils.configurable import parameter
from hackingBuddyGPT.utils.openai.openai_lib import OpenAILib


Prompt = List[Union[ChatCompletionMessage, ChatCompletionMessageParam]]
Context = Any


class WebTestingWithExplanation(Agent):
    llm: OpenAILib
    host: str = parameter(desc="The host to test", default="http://localhost")
    flag_format_description: str = parameter(desc="Description of the flag provided to the LLM", default="a string starting with 'FLAG.' and ending with '.GALF'")
    flag_template: str = parameter(desc="The template of the flags, whereby {flag} is replaced with the flags", default="FLAG.{flag}.GALF")
    flags: str = parameter(desc="A comma (,) separated list of flags to find", default="hostname,dir,username,rootfile,secretfile,adminpass")

    _prompt_history: Prompt = field(default_factory=list)
    _context: Context = field(default_factory=lambda: {"notes": list()})
    _capabilities: Dict[str, Capability] = field(default_factory=dict)
    _all_flags_found: bool = False

    def init(self):
        super().init()
        self._prompt_history.append(
            {
                "role": "system",
                "content": f"You are a senior penetration tester, having received a new project of testing a website at {self.host}."
                           f"Your goal is first to explore the website, building up your knowledge about its parts and inner workings, and then try to plan and execute possible exploitations.\n"
                           f"Think step by step and say everything out loud that you might be doing.\n"
                           f"Give a list of assumptions you base your tool calls on, before you start a tool call, and reason whether they held or not after the call has completed.\n"
                           f"Do not brute-force anything, all information that you might need can be found on the webserver.\n"
                           f"THE MOST IMPORTANT THING TO DO IS, that if you see a flag ({self.flag_format_description}), you should submit it immediately."})
        self._context["host"] = self.host
        self._capabilities = {
            "submit_flag": SubmitFlag(self.flag_format_description, set(self.flag_template.format(flag=flag) for flag in self.flags.split(",")), success_function=self.all_flags_found),
            "http_request": HTTPRequest(self.host),
        }

    def all_flags_found(self):
        self._log.console.print(Panel("All flags found! Congratulations!", title="system"))
        self._all_flags_found = True

    def perform_round(self, turn: int):
        prompt = self._prompt_history  # TODO: in the future, this should do some context truncation

        result: LLMResult = None
        stream = self.llm.stream_response(prompt, self._log.console, capabilities=self._capabilities)
        for part in stream:
            result = part

        message: ChatCompletionMessage = result.result
        message_id = self._log.log_db.add_log_message(self._log.run_id, message.role, message.content, result.tokens_query, result.tokens_response, result.duration)
        self._prompt_history.append(result.result)

        if message.tool_calls is not None:
            for tool_call in message.tool_calls:
                tic = time.perf_counter()
                tool_call_result = self._capabilities[tool_call.function.name].to_model().model_validate_json(tool_call.function.arguments).execute()
                toc = time.perf_counter()

                self._log.console.print(f"\n[bold green on gray3]{' '*self._log.console.width}\nTOOL RESPONSE:[/bold green on gray3]")
                self._log.console.print(tool_call_result)
                self._prompt_history.append(tool_message(tool_call_result, tool_call.id))
                self._log.log_db.add_log_tool_call(self._log.run_id, message_id, tool_call.id, tool_call.function.name, tool_call.function.arguments, tool_call_result, toc - tic)

        return self._all_flags_found


@use_case("Minimal implementation of a web testing use case while allowing the llm to 'talk'")
class WebTestingWithExplanationUseCase(AutonomousAgentUseCase[WebTestingWithExplanation]):
    pass
